@@ -20,7 +20,7 @@
 *  handler with the scanner. In these handler methods, appropriate DOM nodes
 *  are created and added to the DOM tree.
 *
-* $Id: AbstractDOMParser.cpp 678766 2008-07-22 14:00:16Z borisk $
+* $Id: AbstractDOMParser.cpp 901107 2010-01-20 08:45:02Z borisk $
 *
 */
 
@@ -312,6 +312,13 @@ SecurityManager* AbstractDOMParser::getSecurityManager() const
     return fScanner->getSecurityManager();
 }
 
+// Return it as a reference so that we cn return as void* from getParameter.
+//
+const XMLSize_t& AbstractDOMParser::getLowWaterMark() const
+{
+    return fScanner->getLowWaterMark();
+}
+
 bool AbstractDOMParser::getLoadExternalDTD() const
 {
     return fScanner->getLoadExternalDTD();
@@ -445,6 +452,11 @@ void AbstractDOMParser::setSecurityManager(SecurityManager* const securityManage
     fScanner->setSecurityManager(securityManager);
 }
 
+void AbstractDOMParser::setLowWaterMark(XMLSize_t lwm)
+{
+    fScanner->setLowWaterMark(lwm);
+}
+
 void AbstractDOMParser::setLoadExternalDTD(const bool newState)
 {
     fScanner->setLoadExternalDTD(newState);
@@ -859,8 +871,10 @@ void AbstractDOMParser::endElement( const   XMLElementDecl&
     if (fCurrentParent == fDocument)
         fWithinElement = false;
 
-    if(fDoXInclude && XIncludeUtils::isXIIncludeDOMNode(fCurrentNode)
-        || (XIncludeUtils::isXIFallbackDOMNode(fCurrentNode) && !XMLString::equals(fCurrentParent->getNamespaceURI(), XIncludeUtils::fgXIIIncludeNamespaceURI)))
+    if(fDoXInclude &&
+       (XIncludeUtils::isXIIncludeDOMNode(fCurrentNode) ||
+        ((XIncludeUtils::isXIFallbackDOMNode(fCurrentNode) &&
+          !XMLString::equals(fCurrentParent->getNamespaceURI(), XIncludeUtils::fgXIIIncludeNamespaceURI)))))
     {
     	XIncludeUtils xiu((XMLErrorReporter *) this);
 	    // process the XInclude node, then update the fCurrentNode with the new content
@@ -1010,9 +1024,10 @@ void AbstractDOMParser::startElement(const XMLElementDecl&   elemDecl
             //
             unsigned int attrURIId = oneAttrib->getURIId();
             const XMLCh* localName = oneAttrib->getName();
+            const XMLCh* prefix = oneAttrib->getPrefix();
             namespaceURI = 0;
 
-            if (XMLString::equals(localName, XMLUni::fgXMLNSString))
+            if ((prefix==0 || *prefix==0) && XMLString::equals(localName, XMLUni::fgXMLNSString))
             {
                 // xmlns=...
                 attrURIId = xmlnsNSId;
@@ -1024,7 +1039,7 @@ void AbstractDOMParser::startElement(const XMLElementDecl&   elemDecl
             }
 
             attr = (DOMAttrImpl*) createAttrNS (namespaceURI,
-                                                oneAttrib->getPrefix (),
+                                                prefix,
                                                 localName,
                                                 oneAttrib->getQName());
 
