@@ -16,7 +16,7 @@
  */
 
 /*
- * $Id: XMLString.hpp 698579 2008-09-24 14:13:08Z borisk $
+ * $Id: XMLString.hpp 834826 2009-11-11 10:03:53Z borisk $
  */
 
 #if !defined(XERCESC_INCLUDE_GUARD_XMLSTRING_HPP)
@@ -432,9 +432,9 @@ public:
       */
     static bool copyNString
     (
-                XMLCh* const    target
-        , const XMLCh* const    src
-        , const XMLSize_t       maxChars
+                XMLCh* const   target
+        , const XMLCh* const   src
+        , const XMLSize_t      maxChars
     );
     //@}
 
@@ -1020,48 +1020,6 @@ public:
         , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
-#if XERCES_SIZEOF_INT != 8 && XERCES_SIZEOF_LONG != 8 && XERCES_SIZEOF_INT64 != 4
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The number to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * @param manager The MemoryManager to use to allocate objects
-      * will be done
-      */
-    static void binToText
-    (
-        const   XMLUInt64           toFormat
-        ,       char* const         toFill
-        , const XMLSize_t           maxChars
-        , const unsigned int        radix
-        , MemoryManager* const      manager = XMLPlatformUtils::fgMemoryManager
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The number to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * @param manager The MemoryManager to use to allocate objects
-      * will be done
-      */
-    static void binToText
-    (
-        const   XMLUInt64           toFormat
-        ,       XMLCh* const        toFill
-        , const XMLSize_t           maxChars
-        , const unsigned int        radix
-        , MemoryManager* const      manager = XMLPlatformUtils::fgMemoryManager
-    );
-#endif // XERCES_SIZEOF_INT != 8 && XERCES_SIZEOF_LONG != 8 && XERCES_SIZEOF_INT64 != 4
-
     /** Converts binary data to a text string based a given radix
       *
       * @param toFormat The number to convert
@@ -1142,48 +1100,6 @@ public:
         , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
-#if XERCES_SIZEOF_INT != 8 && XERCES_SIZEOF_LONG != 8 && XERCES_SIZEOF_INT64 != 4
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The number to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * @param manager The MemoryManager to use to allocate objects
-      * will be done
-      */
-    static void binToText
-    (
-        const   XMLInt64        toFormat
-        ,       char* const     toFill
-        , const XMLSize_t       maxChars
-        , const unsigned int    radix
-        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The number to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * @param manager The MemoryManager to use to allocate objects
-      * will be done
-      */
-    static void binToText
-    (
-        const   XMLInt64        toFormat
-        ,       XMLCh* const    toFill
-        , const XMLSize_t       maxChars
-        , const unsigned int    radix
-        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
-    );
-#endif // XERCES_SIZEOF_INT != 8 && XERCES_SIZEOF_LONG != 8 && XERCES_SIZEOF_INT64 != 4
-
     /**
       * Converts a string of decimal chars to a binary value
       *
@@ -1417,7 +1333,7 @@ public:
       *        On return , this buffer also holds the converted string
       * @param manager The MemoryManager to use to allocate objects
       */
-    static void replaceWS(XMLCh* const toConvert
+    static void replaceWS(XMLCh* toConvert
         , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager);
 
 	/** Collapse whitespace
@@ -1425,7 +1341,7 @@ public:
       *        On return , this buffer also holds the converted string
       * @param manager The MemoryManager to use to allocate objects
       */
-    static void collapseWS(XMLCh* const toConvert
+    static void collapseWS(XMLCh* toConvert
         , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager);
 
     /** Remove whitespace
@@ -1433,7 +1349,7 @@ public:
       *        On return , this buffer also holds the converted string
       * @param manager The MemoryManager to use to allocate objects
       */
-    static void removeWS(XMLCh* const toConvert
+    static void removeWS(XMLCh* toConvert
     , MemoryManager*       const manager = XMLPlatformUtils::fgMemoryManager);
 
 
@@ -1532,19 +1448,14 @@ inline void XMLString::moveChars(       XMLCh* const targetStr
 
 inline XMLSize_t XMLString::stringLen(const XMLCh* const src)
 {
-    if (src == 0 || *src == 0)
-    {
+    if (src == 0)
         return 0;
-    }
-    else
-    {
-        const XMLCh* pszTmp = src + 1;
 
-        while (*pszTmp)
-            ++pszTmp;
+    const XMLCh* pszTmp = src;
 
-        return (pszTmp - src);
-    }
+    while (*pszTmp++) ;
+
+    return (pszTmp - src - 1);
 }
 
 inline XMLCh* XMLString::replicate(const XMLCh* const toRep,
